@@ -1,15 +1,30 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <src/Channel.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <gee/collection.h>
-#include <gee/arraylist.h>
 #include "src/Schedule.h"
 
 
 
 
 struct _DVBChannelPrivate {
+	guint _GroupId;
 	char* _Name;
 	guint _TransportStreamId;
 	char* _Network;
@@ -26,6 +41,7 @@ struct _DVBChannelPrivate {
 enum  {
 	DVB_CHANNEL_DUMMY_PROPERTY,
 	DVB_CHANNEL_SID,
+	DVB_CHANNEL_GROUP_ID,
 	DVB_CHANNEL_NAME,
 	DVB_CHANNEL_TRANSPORT_STREAM_ID,
 	DVB_CHANNEL_NETWORK,
@@ -38,6 +54,7 @@ enum  {
 };
 static gboolean dvb_channel_real_is_valid (DVBChannel* self);
 static gboolean dvb_channel_real_on_same_transport_stream (DVBChannel* self, DVBChannel* channel);
+static gboolean dvb_channel_real_equals (DVBChannel* self, DVBChannel* channel);
 static void dvb_channel_real_setup_dvb_source (DVBChannel* self, GstElement* source);
 static char* dvb_channel_real_to_string (DVBChannel* self);
 static guint* _uint_dup (guint* self);
@@ -140,6 +157,21 @@ gboolean dvb_channel_on_same_transport_stream (DVBChannel* self, DVBChannel* cha
 }
 
 
+/**
+         * @returns: TRUE of both channels are identical
+         */
+static gboolean dvb_channel_real_equals (DVBChannel* self, DVBChannel* channel) {
+	g_return_val_if_fail (self != NULL, FALSE);
+	g_return_val_if_fail (channel != NULL, FALSE);
+	return self->priv->sid == dvb_channel_get_Sid (channel);
+}
+
+
+gboolean dvb_channel_equals (DVBChannel* self, DVBChannel* channel) {
+	return DVB_CHANNEL_GET_CLASS (self)->equals (self, channel);
+}
+
+
 static void dvb_channel_real_setup_dvb_source (DVBChannel* self, GstElement* source) {
 	g_return_if_fail (self != NULL);
 	g_critical ("Type `%s' does not implement abstract method `dvb_channel_setup_dvb_source'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
@@ -185,6 +217,19 @@ void dvb_channel_set_Sid (DVBChannel* self, guint value) {
 }
 
 
+guint dvb_channel_get_GroupId (DVBChannel* self) {
+	g_return_val_if_fail (self != NULL, 0U);
+	return self->priv->_GroupId;
+}
+
+
+void dvb_channel_set_GroupId (DVBChannel* self, guint value) {
+	g_return_if_fail (self != NULL);
+	self->priv->_GroupId = value;
+	g_object_notify ((GObject *) self, "GroupId");
+}
+
+
 const char* dvb_channel_get_Name (DVBChannel* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_Name;
@@ -344,6 +389,9 @@ static void dvb_channel_get_property (GObject * object, guint property_id, GValu
 		case DVB_CHANNEL_SID:
 		g_value_set_uint (value, dvb_channel_get_Sid (self));
 		break;
+		case DVB_CHANNEL_GROUP_ID:
+		g_value_set_uint (value, dvb_channel_get_GroupId (self));
+		break;
 		case DVB_CHANNEL_NAME:
 		g_value_set_string (value, dvb_channel_get_Name (self));
 		break;
@@ -385,6 +433,9 @@ static void dvb_channel_set_property (GObject * object, guint property_id, const
 		case DVB_CHANNEL_SID:
 		dvb_channel_set_Sid (self, g_value_get_uint (value));
 		break;
+		case DVB_CHANNEL_GROUP_ID:
+		dvb_channel_set_GroupId (self, g_value_get_uint (value));
+		break;
 		case DVB_CHANNEL_NAME:
 		dvb_channel_set_Name (self, g_value_get_string (value));
 		break;
@@ -425,9 +476,11 @@ static void dvb_channel_class_init (DVBChannelClass * klass) {
 	G_OBJECT_CLASS (klass)->finalize = dvb_channel_finalize;
 	DVB_CHANNEL_CLASS (klass)->is_valid = dvb_channel_real_is_valid;
 	DVB_CHANNEL_CLASS (klass)->on_same_transport_stream = dvb_channel_real_on_same_transport_stream;
+	DVB_CHANNEL_CLASS (klass)->equals = dvb_channel_real_equals;
 	DVB_CHANNEL_CLASS (klass)->setup_dvb_source = dvb_channel_real_setup_dvb_source;
 	DVB_CHANNEL_CLASS (klass)->to_string = dvb_channel_real_to_string;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_SID, g_param_spec_uint ("Sid", "Sid", "Sid", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_GROUP_ID, g_param_spec_uint ("GroupId", "GroupId", "GroupId", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_NAME, g_param_spec_string ("Name", "Name", "Name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_TRANSPORT_STREAM_ID, g_param_spec_uint ("TransportStreamId", "TransportStreamId", "TransportStreamId", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_NETWORK, g_param_spec_string ("Network", "Network", "Network", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
