@@ -1,8 +1,26 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <src/ChannelListReader.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gee/collection.h>
+#include <gee.h>
 #include "src/Utils.h"
 #include "src/Channel.h"
 #include "src/Terrestrial/TerrestrialChannel.h"
@@ -16,21 +34,24 @@
 struct _DVBChannelListReaderPrivate {
 	GFile* _ChannelFile;
 	DVBAdapterType _Type;
+	guint _GroupId;
 };
 
 #define DVB_CHANNEL_LIST_READER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), DVB_TYPE_CHANNEL_LIST_READER, DVBChannelListReaderPrivate))
 enum  {
 	DVB_CHANNEL_LIST_READER_DUMMY_PROPERTY,
 	DVB_CHANNEL_LIST_READER_CHANNEL_FILE,
-	DVB_CHANNEL_LIST_READER_TYPE
+	DVB_CHANNEL_LIST_READER_TYPE,
+	DVB_CHANNEL_LIST_READER_GROUP_ID
 };
 static DVBChannel* dvb_channel_list_reader_parse_line (DVBChannelListReader* self, const char* line);
-static DVBTerrestrialChannel* dvb_channel_list_reader_parse_terrestrial_channel (const char* line);
-static DVBSatelliteChannel* dvb_channel_list_reader_parse_satellite_channel (const char* line);
-static DVBCableChannel* dvb_channel_list_reader_parse_cable_channel (const char* line);
+static DVBTerrestrialChannel* dvb_channel_list_reader_parse_terrestrial_channel (DVBChannelListReader* self, const char* line);
+static DVBSatelliteChannel* dvb_channel_list_reader_parse_satellite_channel (DVBChannelListReader* self, const char* line);
+static DVBCableChannel* dvb_channel_list_reader_parse_cable_channel (DVBChannelListReader* self, const char* line);
 static gboolean dvb_channel_list_reader_get_value_with_prefix (GType enumtype, const char* name, const char* prefix, gint* val);
 static void dvb_channel_list_reader_set_ChannelFile (DVBChannelListReader* self, GFile* value);
 static void dvb_channel_list_reader_set_Type (DVBChannelListReader* self, DVBAdapterType value);
+static void dvb_channel_list_reader_set_GroupId (DVBChannelListReader* self, guint value);
 static gpointer dvb_channel_list_reader_parent_class = NULL;
 static void dvb_channel_list_reader_finalize (GObject* obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
@@ -38,12 +59,12 @@ static gint _vala_array_length (gpointer array);
 
 
 
-DVBChannelListReader* dvb_channel_list_reader_construct (GType object_type, GFile* file, DVBAdapterType type) {
+DVBChannelListReader* dvb_channel_list_reader_construct (GType object_type, GFile* file, DVBAdapterType type, guint group_id) {
 	GParameter * __params;
 	GParameter * __params_it;
 	DVBChannelListReader * self;
 	g_return_val_if_fail (file != NULL, NULL);
-	__params = g_new0 (GParameter, 2);
+	__params = g_new0 (GParameter, 3);
 	__params_it = __params;
 	__params_it->name = "ChannelFile";
 	g_value_init (&__params_it->value, G_TYPE_FILE);
@@ -53,6 +74,10 @@ DVBChannelListReader* dvb_channel_list_reader_construct (GType object_type, GFil
 	g_value_init (&__params_it->value, DVB_TYPE_ADAPTER_TYPE);
 	g_value_set_enum (&__params_it->value, type);
 	__params_it++;
+	__params_it->name = "GroupId";
+	g_value_init (&__params_it->value, G_TYPE_UINT);
+	g_value_set_uint (&__params_it->value, group_id);
+	__params_it++;
 	self = g_object_newv (object_type, __params_it - __params, __params);
 	while (__params_it > __params) {
 		--__params_it;
@@ -63,8 +88,8 @@ DVBChannelListReader* dvb_channel_list_reader_construct (GType object_type, GFil
 }
 
 
-DVBChannelListReader* dvb_channel_list_reader_new (GFile* file, DVBAdapterType type) {
-	return dvb_channel_list_reader_construct (DVB_TYPE_CHANNEL_LIST_READER, file, type);
+DVBChannelListReader* dvb_channel_list_reader_new (GFile* file, DVBAdapterType type, guint group_id) {
+	return dvb_channel_list_reader_construct (DVB_TYPE_CHANNEL_LIST_READER, file, type, group_id);
 }
 
 
@@ -106,7 +131,7 @@ DVBChannelList* dvb_channel_list_reader_read (DVBChannelListReader* self, GError
 					if (c != NULL) {
 						dvb_channel_list_add (channels, c);
 					} else {
-						g_warning ("ChannelListReader.vala:27: Could not parse channel");
+						g_warning ("ChannelListReader.vala:48: Could not parse channel");
 					}
 					(c == NULL) ? NULL : (c = (g_object_unref (c), NULL));
 				}
@@ -131,26 +156,26 @@ static DVBChannel* dvb_channel_list_reader_parse_line (DVBChannelListReader* sel
 		{
 			DVBChannel* _tmp0;
 			_tmp0 = NULL;
-			c = (_tmp0 = (DVBChannel*) dvb_channel_list_reader_parse_terrestrial_channel (line), (c == NULL) ? NULL : (c = (g_object_unref (c), NULL)), _tmp0);
+			c = (_tmp0 = (DVBChannel*) dvb_channel_list_reader_parse_terrestrial_channel (self, line), (c == NULL) ? NULL : (c = (g_object_unref (c), NULL)), _tmp0);
 			break;
 		}
 		case DVB_ADAPTER_TYPE_DVB_S:
 		{
 			DVBChannel* _tmp1;
 			_tmp1 = NULL;
-			c = (_tmp1 = (DVBChannel*) dvb_channel_list_reader_parse_satellite_channel (line), (c == NULL) ? NULL : (c = (g_object_unref (c), NULL)), _tmp1);
+			c = (_tmp1 = (DVBChannel*) dvb_channel_list_reader_parse_satellite_channel (self, line), (c == NULL) ? NULL : (c = (g_object_unref (c), NULL)), _tmp1);
 			break;
 		}
 		case DVB_ADAPTER_TYPE_DVB_C:
 		{
 			DVBChannel* _tmp2;
 			_tmp2 = NULL;
-			c = (_tmp2 = (DVBChannel*) dvb_channel_list_reader_parse_cable_channel (line), (c == NULL) ? NULL : (c = (g_object_unref (c), NULL)), _tmp2);
+			c = (_tmp2 = (DVBChannel*) dvb_channel_list_reader_parse_cable_channel (self, line), (c == NULL) ? NULL : (c = (g_object_unref (c), NULL)), _tmp2);
 			break;
 		}
 		default:
 		{
-			g_critical ("ChannelListReader.vala:50: Unknown adapter type");
+			g_critical ("ChannelListReader.vala:71: Unknown adapter type");
 			break;
 		}
 	}
@@ -179,7 +204,7 @@ static DVBChannel* dvb_channel_list_reader_parse_line (DVBChannelListReader* sel
 		}
 		_tmp8 = NULL;
 		val = (_tmp8 = _tmp5, (_tmp8 == NULL) ? NULL : g_strdup (_tmp8));
-		g_warning ("ChannelListReader.vala:58: Channel is not valid: %s", val);
+		g_warning ("ChannelListReader.vala:79: Channel is not valid: %s", val);
 		_tmp9 = NULL;
 		return (_tmp9 = NULL, _tmp5 = (g_free (_tmp5), NULL), val = (g_free (val), NULL), (c == NULL) ? NULL : (c = (g_object_unref (c), NULL)), _tmp9);
 	}
@@ -194,7 +219,7 @@ static DVBChannel* dvb_channel_list_reader_parse_line (DVBChannelListReader* sel
          * A line looks like
          * Das Erste:212500000:INVERSION_AUTO:BANDWIDTH_7_MHZ:FEC_3_4:FEC_1_2:QAM_16:TRANSMISSION_MODE_8K:GUARD_INTERVAL_1_4:HIERARCHY_NONE:513:514:32
          */
-static DVBTerrestrialChannel* dvb_channel_list_reader_parse_terrestrial_channel (const char* line) {
+static DVBTerrestrialChannel* dvb_channel_list_reader_parse_terrestrial_channel (DVBChannelListReader* self, const char* line) {
 	DVBTerrestrialChannel* channel;
 	char** _tmp1;
 	gint fields_size;
@@ -206,8 +231,10 @@ static DVBTerrestrialChannel* dvb_channel_list_reader_parse_terrestrial_channel
 	gboolean failed;
 	char* _tmp3;
 	const char* _tmp2;
+	g_return_val_if_fail (self != NULL, NULL);
 	g_return_val_if_fail (line != NULL, NULL);
 	channel = dvb_terrestrial_channel_new ();
+	dvb_channel_set_GroupId ((DVBChannel*) channel, self->priv->_GroupId);
 	_tmp1 = NULL;
 	_tmp0 = NULL;
 	fields = (_tmp1 = _tmp0 = g_strsplit (line, ":", 0), fields_length1 = _vala_array_length (_tmp0), fields_size = fields_length1, _tmp1);
@@ -221,7 +248,7 @@ static DVBTerrestrialChannel* dvb_channel_list_reader_parse_terrestrial_channel
 			if (g_utf8_validate (val, -1, NULL)) {
 				dvb_channel_set_Name ((DVBChannel*) channel, val);
 			} else {
-				g_warning ("ChannelListReader.vala:83: Bad UTF-8 encoded channel name");
+				g_warning ("ChannelListReader.vala:105: Bad UTF-8 encoded channel name");
 				dvb_channel_set_Name ((DVBChannel*) channel, "Bad encoding");
 			}
 		} else {
@@ -351,7 +378,7 @@ static DVBTerrestrialChannel* dvb_channel_list_reader_parse_terrestrial_channel
          * A line looks like
          * Das Erste:11836:h:0:27500:101:102:28106
          */
-static DVBSatelliteChannel* dvb_channel_list_reader_parse_satellite_channel (const char* line) {
+static DVBSatelliteChannel* dvb_channel_list_reader_parse_satellite_channel (DVBChannelListReader* self, const char* line) {
 	DVBSatelliteChannel* channel;
 	char** _tmp1;
 	gint fields_size;
@@ -363,8 +390,10 @@ static DVBSatelliteChannel* dvb_channel_list_reader_parse_satellite_channel (con
 	char* _tmp3;
 	const char* _tmp2;
 	DVBSatelliteChannel* _tmp4;
+	g_return_val_if_fail (self != NULL, NULL);
 	g_return_val_if_fail (line != NULL, NULL);
 	channel = dvb_satellite_channel_new ();
+	dvb_channel_set_GroupId ((DVBChannel*) channel, self->priv->_GroupId);
 	_tmp1 = NULL;
 	_tmp0 = NULL;
 	fields = (_tmp1 = _tmp0 = g_strsplit (line, ":", 0), fields_length1 = _vala_array_length (_tmp0), fields_size = fields_length1, _tmp1);
@@ -377,7 +406,7 @@ static DVBSatelliteChannel* dvb_channel_list_reader_parse_satellite_channel (con
 			if (g_utf8_validate (val, -1, NULL)) {
 				dvb_channel_set_Name ((DVBChannel*) channel, val);
 			} else {
-				g_warning ("ChannelListReader.vala:192: Bad UTF-8 encoded channel name");
+				g_warning ("ChannelListReader.vala:215: Bad UTF-8 encoded channel name");
 				dvb_channel_set_Name ((DVBChannel*) channel, "Bad encoding");
 			}
 		} else {
@@ -424,7 +453,7 @@ static DVBSatelliteChannel* dvb_channel_list_reader_parse_satellite_channel (con
          * line looks like
          * ProSieben:330000000:INVERSION_AUTO:6900000:FEC_NONE:QAM_64:255:256:898
          */
-static DVBCableChannel* dvb_channel_list_reader_parse_cable_channel (const char* line) {
+static DVBCableChannel* dvb_channel_list_reader_parse_cable_channel (DVBChannelListReader* self, const char* line) {
 	DVBCableChannel* channel;
 	char** _tmp1;
 	gint fields_size;
@@ -436,8 +465,10 @@ static DVBCableChannel* dvb_channel_list_reader_parse_cable_channel (const char*
 	gboolean failed;
 	char* _tmp3;
 	const char* _tmp2;
+	g_return_val_if_fail (self != NULL, NULL);
 	g_return_val_if_fail (line != NULL, NULL);
 	channel = dvb_cable_channel_new ();
+	dvb_channel_set_GroupId ((DVBChannel*) channel, self->priv->_GroupId);
 	_tmp1 = NULL;
 	_tmp0 = NULL;
 	fields = (_tmp1 = _tmp0 = g_strsplit (line, ":", 0), fields_length1 = _vala_array_length (_tmp0), fields_size = fields_length1, _tmp1);
@@ -451,7 +482,7 @@ static DVBCableChannel* dvb_channel_list_reader_parse_cable_channel (const char*
 			if (g_utf8_validate (val, -1, NULL)) {
 				dvb_channel_set_Name ((DVBChannel*) channel, val);
 			} else {
-				g_warning ("ChannelListReader.vala:238: Bad UTF-8 encoded channel name");
+				g_warning ("ChannelListReader.vala:262: Bad UTF-8 encoded channel name");
 				dvb_channel_set_Name ((DVBChannel*) channel, "Bad encoding");
 			}
 		} else {
@@ -565,6 +596,19 @@ static void dvb_channel_list_reader_set_Type (DVBChannelListReader* self, DVBAda
 }
 
 
+guint dvb_channel_list_reader_get_GroupId (DVBChannelListReader* self) {
+	g_return_val_if_fail (self != NULL, 0U);
+	return self->priv->_GroupId;
+}
+
+
+static void dvb_channel_list_reader_set_GroupId (DVBChannelListReader* self, guint value) {
+	g_return_if_fail (self != NULL);
+	self->priv->_GroupId = value;
+	g_object_notify ((GObject *) self, "GroupId");
+}
+
+
 static void dvb_channel_list_reader_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
 	DVBChannelListReader * self;
 	gpointer boxed;
@@ -576,6 +620,9 @@ static void dvb_channel_list_reader_get_property (GObject * object, guint proper
 		case DVB_CHANNEL_LIST_READER_TYPE:
 		g_value_set_enum (value, dvb_channel_list_reader_get_Type (self));
 		break;
+		case DVB_CHANNEL_LIST_READER_GROUP_ID:
+		g_value_set_uint (value, dvb_channel_list_reader_get_GroupId (self));
+		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -593,6 +640,9 @@ static void dvb_channel_list_reader_set_property (GObject * object, guint proper
 		case DVB_CHANNEL_LIST_READER_TYPE:
 		dvb_channel_list_reader_set_Type (self, g_value_get_enum (value));
 		break;
+		case DVB_CHANNEL_LIST_READER_GROUP_ID:
+		dvb_channel_list_reader_set_GroupId (self, g_value_get_uint (value));
+		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -608,6 +658,7 @@ static void dvb_channel_list_reader_class_init (DVBChannelListReaderClass * klas
 	G_OBJECT_CLASS (klass)->finalize = dvb_channel_list_reader_finalize;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_LIST_READER_CHANNEL_FILE, g_param_spec_object ("ChannelFile", "ChannelFile", "ChannelFile", G_TYPE_FILE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_LIST_READER_TYPE, g_param_spec_enum ("Type", "Type", "Type", DVB_TYPE_ADAPTER_TYPE, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_LIST_READER_GROUP_ID, g_param_spec_uint ("GroupId", "GroupId", "GroupId", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 
