@@ -1,9 +1,26 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <src/ChannelListWriter.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gee/collection.h>
-#include <gee/list.h>
+#include <gee.h>
 #include "src/Terrestrial/TerrestrialChannel.h"
 #include "src/Utils.h"
 #include "src/MpegTsEnums.h"
@@ -172,7 +189,7 @@ void dvb_channel_list_writer_write (DVBChannelListWriter* self, DVBChannel* chan
 					return;
 				}
 			} else {
-				g_warning ("ChannelListWriter.vala:57: Unknown channel type");
+				g_warning ("ChannelListWriter.vala:76: Unknown channel type");
 			}
 		}
 	}
