@@ -1,8 +1,23 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <src/DeviceGroup.h>
-#include <gee/set.h>
-#include <gee/collection.h>
-#include <gee/hashset.h>
 #include "src/EPGScanner.h"
 #include "src/Recorder.h"
 
@@ -45,15 +60,16 @@ static void dvb_device_group_finalize (GObject* obj);
 
 
 /**
+         * @id: ID of group
          * @reference_device: All devices of this group will inherit
          * the settings from this device
+         * @with_epg_scanner: Whether to provide an EPG scanner
          */
-DVBDeviceGroup* dvb_device_group_construct (GType object_type, guint id, DVBDevice* reference_device) {
+DVBDeviceGroup* dvb_device_group_construct (GType object_type, guint id, DVBDevice* reference_device, gboolean with_epg_scanner) {
 	GParameter * __params;
 	GParameter * __params_it;
 	DVBDeviceGroup * self;
-	DVBEPGScanner* _tmp0;
-	DVBRecorder* _tmp1;
+	DVBRecorder* _tmp2;
 	g_return_val_if_fail (reference_device != NULL, NULL);
 	__params = g_new0 (GParameter, 2);
 	__params_it = __params;
@@ -66,11 +82,17 @@ DVBDeviceGroup* dvb_device_group_construct (GType object_type, guint id, DVBDevi
 	g_value_set_object (&__params_it->value, reference_device);
 	__params_it++;
 	self = g_object_newv (object_type, __params_it - __params, __params);
-	dvb_channel_list_set_group_id (dvb_device_get_Channels (self->priv->_reference_device), id);
-	_tmp0 = NULL;
-	self->priv->_epgscanner = (_tmp0 = dvb_epg_scanner_new (self), (self->priv->_epgscanner == NULL) ? NULL : (self->priv->_epgscanner = (g_object_unref (self->priv->_epgscanner), NULL)), _tmp0);
-	_tmp1 = NULL;
-	self->priv->_recorder = (_tmp1 = dvb_recorder_new (self), (self->priv->_recorder == NULL) ? NULL : (self->priv->_recorder = (g_object_unref (self->priv->_recorder), NULL)), _tmp1);
+	if (with_epg_scanner) {
+		DVBEPGScanner* _tmp0;
+		_tmp0 = NULL;
+		self->priv->_epgscanner = (_tmp0 = dvb_epg_scanner_new (self), (self->priv->_epgscanner == NULL) ? NULL : (self->priv->_epgscanner = (g_object_unref (self->priv->_epgscanner), NULL)), _tmp0);
+	} else {
+		DVBEPGScanner* _tmp1;
+		_tmp1 = NULL;
+		self->priv->_epgscanner = (_tmp1 = NULL, (self->priv->_epgscanner == NULL) ? NULL : (self->priv->_epgscanner = (g_object_unref (self->priv->_epgscanner), NULL)), _tmp1);
+	}
+	_tmp2 = NULL;
+	self->priv->_recorder = (_tmp2 = dvb_recorder_new (self), (self->priv->_recorder == NULL) ? NULL : (self->priv->_recorder = (g_object_unref (self->priv->_recorder), NULL)), _tmp2);
 	while (__params_it > __params) {
 		--__params_it;
 		g_value_unset (&__params_it->value);
@@ -80,15 +102,17 @@ DVBDeviceGroup* dvb_device_group_construct (GType object_type, guint id, DVBDevi
 }
 
 
-DVBDeviceGroup* dvb_device_group_new (guint id, DVBDevice* reference_device) {
-	return dvb_device_group_construct (DVB_TYPE_DEVICE_GROUP, id, reference_device);
+DVBDeviceGroup* dvb_device_group_new (guint id, DVBDevice* reference_device, gboolean with_epg_scanner) {
+	return dvb_device_group_construct (DVB_TYPE_DEVICE_GROUP, id, reference_device, with_epg_scanner);
 }
 
 
 void dvb_device_group_destroy (DVBDeviceGroup* self) {
 	g_return_if_fail (self != NULL);
-	g_debug ("DeviceGroup.vala:58: Destroying group %u", self->priv->_Id);
-	dvb_epg_scanner_destroy (self->priv->_epgscanner);
+	g_debug ("DeviceGroup.vala:83: Destroying group %u", self->priv->_Id);
+	if (self->priv->_epgscanner != NULL) {
+		dvb_epg_scanner_destroy (self->priv->_epgscanner);
+	}
 	dvb_recorder_stop (self->priv->_recorder);
 }
 
@@ -118,7 +142,7 @@ gboolean dvb_device_group_add (DVBDeviceGroup* self, DVBDevice* device) {
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (device != NULL, FALSE);
 	if (dvb_device_get_Type (device) != dvb_device_group_get_Type (self)) {
-		g_warning ("DeviceGroup.vala:82: Cannot add device, because it is not of same type");
+		g_warning ("DeviceGroup.vala:108: Cannot add device, because it is not of same type");
 		return FALSE;
 	}
 	/* Set settings from reference device*/
