@@ -1,12 +1,29 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 #define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
 
 #include <src/EPGScanner.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gee/hashset.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <gee/collection.h>
+#include <gee.h>
+#include "src/Factory.h"
+#include "src/Settings.h"
 #include "src/Channel.h"
 #include "src/Device.h"
 #include "src/ChannelList.h"
@@ -31,7 +48,7 @@ enum  {
 	DVB_EPG_SCANNER_DUMMY_PROPERTY,
 	DVB_EPG_SCANNER_DEVICE_GROUP
 };
-#define DVB_EPG_SCANNER_CHECK_EIT_INTERVAL 15 * 60
+static gint dvb_epg_scanner_CHECK_EIT_INTERVAL = 0;
 #define DVB_EPG_SCANNER_WAIT_FOR_EIT_DURATION 10
 #define DVB_EPG_SCANNER_PIPELINE_TEMPLATE "dvbsrc name=dvbsrc adapter=%u frontend=%u pids=0:16:17:18 stats-reporting-interval=0 ! mpegtsparse ! fakesink silent=true"
 static void dvb_epg_scanner_remove_timeouts (DVBEPGScanner* self);
@@ -53,10 +70,49 @@ static int _vala_strcmp0 (const char * str1, const char * str2);
          * @device: The device where EPG should be collected from
          */
 DVBEPGScanner* dvb_epg_scanner_construct (GType object_type, DVBDeviceGroup* device) {
+	GError * inner_error;
 	DVBEPGScanner * self;
 	g_return_val_if_fail (device != NULL, NULL);
+	inner_error = NULL;
 	self = g_object_newv (object_type, 0, NULL);
 	dvb_epg_scanner_set_DeviceGroup (self, device);
+	if (dvb_epg_scanner_CHECK_EIT_INTERVAL == (-1)) {
+		DVBSettings* _tmp0;
+		DVBSettings* settings;
+		_tmp0 = NULL;
+		settings = (_tmp0 = dvb_factory_get_settings (), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
+		{
+			gint _tmp1;
+			_tmp1 = dvb_settings_get_integer (settings, DVB_SETTINGS_EPG_SECTION, DVB_SETTINGS_SCAN_INTERVAL, &inner_error);
+			if (inner_error != NULL) {
+				if (inner_error->domain == G_KEY_FILE_ERROR) {
+					goto __catch3_g_key_file_error;
+				}
+				goto __finally3;
+			}
+			dvb_epg_scanner_CHECK_EIT_INTERVAL = _tmp1 * 60;
+		}
+		goto __finally3;
+		__catch3_g_key_file_error:
+		{
+			GError * e;
+			e = inner_error;
+			inner_error = NULL;
+			{
+				g_critical ("EPGScanner.vala:62: %s", e->message);
+				dvb_epg_scanner_CHECK_EIT_INTERVAL = 15 * 60;
+				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
+			}
+		}
+		__finally3:
+		if (inner_error != NULL) {
+			(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
+			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
+			g_clear_error (&inner_error);
+			return NULL;
+		}
+		(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
+	}
 	return self;
 }
 
@@ -71,7 +127,7 @@ DVBEPGScanner* dvb_epg_scanner_new (DVBDeviceGroup* device) {
          */
 void dvb_epg_scanner_stop (DVBEPGScanner* self) {
 	g_return_if_fail (self != NULL);
-	g_debug ("EPGScanner.vala:42: Stopping EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
+	g_debug ("EPGScanner.vala:72: Stopping EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
 	dvb_epg_scanner_remove_timeouts (self);
 	dvb_epg_scanner_reset (self);
 }
@@ -149,7 +205,7 @@ gboolean dvb_epg_scanner_start (DVBEPGScanner* self) {
 	gboolean _tmp7;
 	g_return_val_if_fail (self != NULL, FALSE);
 	inner_error = NULL;
-	g_debug ("EPGScanner.vala:90: Starting EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
+	g_debug ("EPGScanner.vala:120: Starting EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
 	/* TODO scan all channels?*/
 	unique_frequencies = gee_hash_set_new (G_TYPE_UINT, NULL, NULL, g_direct_hash, g_direct_equal);
 	{
@@ -187,25 +243,25 @@ gboolean dvb_epg_scanner_start (DVBEPGScanner* self) {
 			_tmp3 = NULL;
 			_tmp4 = (_tmp3 = gst_parse_launch (_tmp2 = g_strdup_printf (DVB_EPG_SCANNER_PIPELINE_TEMPLATE, dvb_device_get_Adapter (device), dvb_device_get_Frontend (device)), &inner_error), _tmp2 = (g_free (_tmp2), NULL), _tmp3);
 			if (inner_error != NULL) {
-				goto __catch3_g_error;
-				goto __finally3;
+				goto __catch4_g_error;
+				goto __finally4;
 			}
 			_tmp5 = NULL;
 			self->priv->pipeline = (_tmp5 = _tmp4, (self->priv->pipeline == NULL) ? NULL : (self->priv->pipeline = (gst_object_unref (self->priv->pipeline), NULL)), _tmp5);
 		}
-		goto __finally3;
-		__catch3_g_error:
+		goto __finally4;
+		__catch4_g_error:
 		{
 			GError * e;
 			e = inner_error;
 			inner_error = NULL;
 			{
 				gboolean _tmp6;
-				g_error ("EPGScanner.vala:110: Could not create pipeline: %s", e->message);
+				g_error ("EPGScanner.vala:140: Could not create pipeline: %s", e->message);
 				return (_tmp6 = FALSE, (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), (unique_frequencies == NULL) ? NULL : (unique_frequencies = (g_object_unref (unique_frequencies), NULL)), (device == NULL) ? NULL : (device = (g_object_unref (device), NULL)), _tmp6);
 			}
 		}
-		__finally3:
+		__finally4:
 		if (inner_error != NULL) {
 			(unique_frequencies == NULL) ? NULL : (unique_frequencies = (g_object_unref (unique_frequencies), NULL));
 			(device == NULL) ? NULL : (device = (g_object_unref (device), NULL));
@@ -244,10 +300,10 @@ static gboolean dvb_epg_scanner_scan_new_frequency (DVBEPGScanner* self) {
 		_tmp0 = self->priv->do_stop;
 	}
 	if (_tmp0) {
-		g_debug ("EPGScanner.vala:130: Finished EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
+		g_debug ("EPGScanner.vala:160: Finished EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
 		dvb_epg_scanner_reset (self);
 		/* Time the next iteration*/
-		self->priv->queue_scan_event_id = g_timeout_add_seconds ((guint) DVB_EPG_SCANNER_CHECK_EIT_INTERVAL, _dvb_epg_scanner_start_gsource_func, self);
+		self->priv->queue_scan_event_id = g_timeout_add_seconds ((guint) dvb_epg_scanner_CHECK_EIT_INTERVAL, _dvb_epg_scanner_start_gsource_func, self);
 		return FALSE;
 	}
 	channel = (DVBChannel*) g_queue_pop_head (self->priv->channels);
@@ -274,7 +330,7 @@ static void dvb_epg_scanner_bus_watch_func (DVBEPGScanner* self, GstBus* bus, Gs
 		case GST_MESSAGE_ELEMENT:
 		{
 			if (_vala_strcmp0 (gst_structure_get_name (message->structure), "dvb-read-failure") == 0) {
-				g_critical ("EPGScanner.vala:159: Could not read from DVB device");
+				g_critical ("EPGScanner.vala:189: Could not read from DVB device");
 				dvb_epg_scanner_stop (self);
 			} else {
 				if (_vala_strcmp0 (gst_structure_get_name (message->structure), "eit") == 0) {
@@ -299,7 +355,7 @@ static void dvb_epg_scanner_bus_watch_func (DVBEPGScanner* self, GstBus* bus, Gs
 			_tmp0 = NULL;
 			(gst_message_parse_error (message, &_tmp0, &_tmp2), gerror = (_tmp1 = _tmp0, (gerror == NULL) ? NULL : (gerror = (g_error_free (gerror), NULL)), _tmp1));
 			debug = (_tmp3 = _tmp2, debug = (g_free (debug), NULL), _tmp3);
-			g_critical ("EPGScanner.vala:170: %s %s", gerror->message, debug);
+			g_critical ("EPGScanner.vala:200: %s %s", gerror->message, debug);
 			dvb_epg_scanner_stop (self);
 			(gerror == NULL) ? NULL : (gerror = (g_error_free (gerror), NULL));
 			debug = (g_free (debug), NULL);
@@ -354,7 +410,7 @@ void dvb_epg_scanner_on_eit_structure (DVBEPGScanner* self, const GstStructure*
 			sid = dvb_epg_scanner_get_uint_val (structure, "service-id");
 			channel = dvb_channel_list_get_channel (dvb_device_group_get_Channels (self->priv->_DeviceGroup), sid);
 			if (channel == NULL) {
-				g_warning ("EPGScanner.vala:196: Could not find channel %u for this device", sid);
+				g_warning ("EPGScanner.vala:226: Could not find channel %u for this device", sid);
 				(channel == NULL) ? NULL : (channel = (g_object_unref (channel), NULL));
 				return;
 			}
@@ -573,6 +629,7 @@ static void dvb_epg_scanner_class_init (DVBEPGScannerClass * klass) {
 	G_OBJECT_CLASS (klass)->constructor = dvb_epg_scanner_constructor;
 	G_OBJECT_CLASS (klass)->finalize = dvb_epg_scanner_finalize;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_EPG_SCANNER_DEVICE_GROUP, g_param_spec_object ("DeviceGroup", "DeviceGroup", "DeviceGroup", DVB_TYPE_DEVICE_GROUP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	dvb_epg_scanner_CHECK_EIT_INTERVAL = -1;
 }
 
 
