@@ -1,3 +1,21 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <src/Event.h>
 #include "src/Utils.h"
@@ -178,19 +196,12 @@ static gint64 dvb_event_get_end_timestamp (DVBEvent* self) {
 	struct tm end_time;
 	gint64 before;
 	gint64 after;
-	gboolean _tmp0;
 	g_return_val_if_fail (self != NULL, 0LL);
 	end_time = dvb_utils_create_utc_time ((gint) self->year, (gint) self->month, (gint) self->day, (gint) self->hour, (gint) self->minute, (gint) self->second);
 	before = (gint64) mktime (&end_time);
 	end_time.tm_sec = end_time.tm_sec + ((gint) self->duration);
 	after = (gint64) mktime (&end_time);
-	_tmp0 = FALSE;
-	if (after > before) {
-		_tmp0 = (after - before) == self->duration;
-	} else {
-		_tmp0 = FALSE;
-	}
-	g_assert (_tmp0);
+	g_assert ((after - before) == self->duration);
 	return after;
 }
 
