@@ -1,3 +1,21 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <src/Recording.h>
 #include <stdio.h>
@@ -9,6 +27,7 @@
 struct _DVBRecordingPrivate {
 	guint32 _Id;
 	guint _ChannelSid;
+	char* _ChannelName;
 	GFile* _Location;
 	char* _Name;
 	char* _Description;
@@ -21,6 +40,7 @@ enum  {
 	DVB_RECORDING_DUMMY_PROPERTY,
 	DVB_RECORDING_ID,
 	DVB_RECORDING_CHANNEL_SID,
+	DVB_RECORDING_CHANNEL_NAME,
 	DVB_RECORDING_LOCATION,
 	DVB_RECORDING_NAME,
 	DVB_RECORDING_DESCRIPTION,
@@ -60,10 +80,10 @@ void dvb_recording_save_to_disk (DVBRecording* self, GError** error) {
 	parentdir = g_file_get_parent (self->priv->_Location);
 	recfile = g_file_get_child (parentdir, "info.rec");
 	_tmp0 = NULL;
-	g_debug ("Recording.vala:37: Saving recording to %s", _tmp0 = g_file_get_path (recfile));
+	g_debug ("Recording.vala:57: Saving recording to %s", _tmp0 = g_file_get_path (recfile));
 	_tmp0 = (g_free (_tmp0), NULL);
 	if (g_file_query_exists (recfile, NULL)) {
-		g_debug ("Recording.vala:40: Deleting old info.rec");
+		g_debug ("Recording.vala:60: Deleting old info.rec");
 		g_file_delete (recfile, NULL, &inner_error);
 		if (inner_error != NULL) {
 			g_propagate_error (error, inner_error);
@@ -134,7 +154,7 @@ char* dvb_recording_serialize (DVBRecording* self) {
 	_tmp4 = NULL;
 	_tmp5 = NULL;
 	_tmp6 = NULL;
-	return (_tmp6 = (_tmp5 = g_strdup_printf ("%u\n%u\n%s\n%u-%u-%u %u:%u\n%lld\n%s\n%s", (guint) self->priv->_Id, self->priv->_ChannelSid, _tmp4 = g_file_get_path (self->priv->_Location), started[0], started[1], started[2], started[3], started[4], self->priv->_Length, _tmp2, _tmp3), _tmp4 = (g_free (_tmp4), NULL), _tmp5), started = (g_free (started), NULL), _tmp6);
+	return (_tmp6 = (_tmp5 = g_strdup_printf ("%u\n%s\n%s\n%u-%u-%u %u:%u\n%" G_GINT64_FORMAT "\n%s\n%s", self->priv->_Id, self->priv->_ChannelName, _tmp4 = g_file_get_path (self->priv->_Location), started[0], started[1], started[2], started[3], started[4], self->priv->_Length, _tmp2, _tmp3), _tmp4 = (g_free (_tmp4), NULL), _tmp5), started = (g_free (started), NULL), _tmp6);
 }
 
 
@@ -182,7 +202,7 @@ DVBRecording* dvb_recording_deserialize (GFile* file, GError** error) {
 			}
 			case 1:
 			{
-				dvb_recording_set_ChannelSid (rec, (guint) atoi (field));
+				dvb_recording_set_ChannelName (rec, field);
 				break;
 			}
 			case 2:
@@ -317,6 +337,23 @@ void dvb_recording_set_ChannelSid (DVBRecording* self, guint value) {
 }
 
 
+const char* dvb_recording_get_ChannelName (DVBRecording* self) {
+	g_return_val_if_fail (self != NULL, NULL);
+	return self->priv->_ChannelName;
+}
+
+
+void dvb_recording_set_ChannelName (DVBRecording* self, const char* value) {
+	char* _tmp2;
+	const char* _tmp1;
+	g_return_if_fail (self != NULL);
+	_tmp2 = NULL;
+	_tmp1 = NULL;
+	self->priv->_ChannelName = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_ChannelName = (g_free (self->priv->_ChannelName), NULL), _tmp2);
+	g_object_notify ((GObject *) self, "ChannelName");
+}
+
+
 GFile* dvb_recording_get_Location (DVBRecording* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_Location;
@@ -405,6 +442,9 @@ static void dvb_recording_get_property (GObject * object, guint property_id, GVa
 		case DVB_RECORDING_CHANNEL_SID:
 		g_value_set_uint (value, dvb_recording_get_ChannelSid (self));
 		break;
+		case DVB_RECORDING_CHANNEL_NAME:
+		g_value_set_string (value, dvb_recording_get_ChannelName (self));
+		break;
 		case DVB_RECORDING_LOCATION:
 		g_value_set_object (value, dvb_recording_get_Location (self));
 		break;
@@ -434,6 +474,9 @@ static void dvb_recording_set_property (GObject * object, guint property_id, con
 		case DVB_RECORDING_CHANNEL_SID:
 		dvb_recording_set_ChannelSid (self, g_value_get_uint (value));
 		break;
+		case DVB_RECORDING_CHANNEL_NAME:
+		dvb_recording_set_ChannelName (self, g_value_get_string (value));
+		break;
 		case DVB_RECORDING_LOCATION:
 		dvb_recording_set_Location (self, g_value_get_object (value));
 		break;
@@ -461,6 +504,7 @@ static void dvb_recording_class_init (DVBRecordingClass * klass) {
 	G_OBJECT_CLASS (klass)->finalize = dvb_recording_finalize;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_RECORDING_ID, g_param_spec_uint ("Id", "Id", "Id", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_RECORDING_CHANNEL_SID, g_param_spec_uint ("ChannelSid", "ChannelSid", "ChannelSid", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_RECORDING_CHANNEL_NAME, g_param_spec_string ("ChannelName", "ChannelName", "ChannelName", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_RECORDING_LOCATION, g_param_spec_object ("Location", "Location", "Location", G_TYPE_FILE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_RECORDING_NAME, g_param_spec_string ("Name", "Name", "Name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_RECORDING_DESCRIPTION, g_param_spec_string ("Description", "Description", "Description", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -476,6 +520,7 @@ static void dvb_recording_instance_init (DVBRecording * self) {
 static void dvb_recording_finalize (GObject* obj) {
 	DVBRecording * self;
 	self = DVB_RECORDING (obj);
+	self->priv->_ChannelName = (g_free (self->priv->_ChannelName), NULL);
 	(self->priv->_Location == NULL) ? NULL : (self->priv->_Location = (g_object_unref (self->priv->_Location), NULL));
 	self->priv->_Name = (g_free (self->priv->_Name), NULL);
 	self->priv->_Description = (g_free (self->priv->_Description), NULL);
