@@ -1,3 +1,21 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef __SRC_RECORDING_H__
 #define __SRC_RECORDING_H__
@@ -46,6 +64,8 @@ guint32 dvb_recording_get_Id (DVBRecording* self);
 void dvb_recording_set_Id (DVBRecording* self, guint32 value);
 guint dvb_recording_get_ChannelSid (DVBRecording* self);
 void dvb_recording_set_ChannelSid (DVBRecording* self, guint value);
+const char* dvb_recording_get_ChannelName (DVBRecording* self);
+void dvb_recording_set_ChannelName (DVBRecording* self, const char* value);
 GFile* dvb_recording_get_Location (DVBRecording* self);
 void dvb_recording_set_Location (DVBRecording* self, GFile* value);
 const char* dvb_recording_get_Name (DVBRecording* self);
