@@ -1,3 +1,21 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <src/SqliteEPGStore.h>
 #include <sqlite3.h>
@@ -6,8 +24,6 @@
 #include <float.h>
 #include <math.h>
 #include <stdio.h>
-#include <gee/arraylist.h>
-#include <gee/collection.h>
 #include <gio/gio.h>
 #include "src/Utils.h"
 
@@ -28,19 +44,19 @@ struct _DVBSqliteEPGStorePrivate {
 enum  {
 	DVB_SQLITE_EPG_STORE_DUMMY_PROPERTY
 };
-#define DVB_SQLITE_EPG_STORE_CREATE_EVENTS_TABLE_STATEMENT "CREATE TABLE events (sid INTEGER,\n            event_id INTEGER,\n            starttime JULIAN,\n            duration INTEGER,\n            running_status INTEGER(2),\n            free_ca_mode INTEGER(1),\n            name VARCHAR(255),\n            description VARCHAR(255),\n            extended_description TEXT,\n            PRIMARY KEY (sid, event_id))"
-#define DVB_SQLITE_EPG_STORE_INSERT_EVENT_SQL "INSERT INTO events VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)"
-#define DVB_SQLITE_EPG_STORE_DELETE_EVENT_STATEMENT "DELETE FROM events WHERE sid=? AND event_id=?"
-#define DVB_SQLITE_EPG_STORE_SELECT_ALL_EVENTS_STATEMENT "SELECT event_id, datetime(starttime),\n            duration, running_status, free_ca_mode, name,\n            description, extended_description\n            FROM events WHERE sid='%u'"
-#define DVB_SQLITE_EPG_STORE_HAS_EVENT_STATEMENT "SELECT COUNT(*) FROM events WHERE sid=? AND event_id=?"
-#define DVB_SQLITE_EPG_STORE_UPDATE_EVENT_SQL "UPDATE events SET starttime=?, duration=?, running_status=?,\n            free_ca_mode=?, name=?, description=?,\n            extended_description=? WHERE sid=? AND event_id=?"
+#define DVB_SQLITE_EPG_STORE_CREATE_EVENTS_TABLE_STATEMENT "CREATE TABLE events (group_id INTEGER,\n            sid INTEGER,\n            event_id INTEGER,\n            starttime JULIAN,\n            duration INTEGER,\n            running_status INTEGER(2),\n            free_ca_mode INTEGER(1),\n            name VARCHAR(255),\n            description VARCHAR(255),\n            extended_description TEXT,\n            PRIMARY KEY (sid, event_id))"
+#define DVB_SQLITE_EPG_STORE_INSERT_EVENT_SQL "INSERT INTO events VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"
+#define DVB_SQLITE_EPG_STORE_DELETE_EVENT_STATEMENT "DELETE FROM events WHERE group_id=? AND sid=? AND event_id=?"
+#define DVB_SQLITE_EPG_STORE_SELECT_ALL_EVENTS_STATEMENT "SELECT event_id, datetime(starttime),\n            duration, running_status, free_ca_mode, name,\n            description, extended_description\n            FROM events WHERE group_id='%u' AND sid='%u'"
+#define DVB_SQLITE_EPG_STORE_HAS_EVENT_STATEMENT "SELECT COUNT(*) FROM events WHERE group_id=? AND sid=? AND event_id=?"
+#define DVB_SQLITE_EPG_STORE_UPDATE_EVENT_SQL "UPDATE events SET starttime=?, duration=?, running_status=?,\n            free_ca_mode=?, name=?, description=?,\n            extended_description=? WHERE group_id=? AND sid=? AND event_id=?"
 #define DVB_SQLITE_EPG_STORE_TO_JULIAN_SQL "SELECT julianday(?)"
-#define DVB_SQLITE_EPG_STORE_SELECT_EVENT_SQL "SELECT event_id, datetime(starttime),\n            duration, running_status, free_ca_mode, name,\n            description, extended_description\n            FROM events WHERE sid=? AND event_id=?"
-static gboolean dvb_sqlite_epg_store_real_add_or_update_event (DVBEPGStore* base, DVBEvent* event, DVBChannel* channel);
-static DVBEvent* dvb_sqlite_epg_store_real_get_event (DVBEPGStore* base, guint event_id, guint channel_sid);
-static gboolean dvb_sqlite_epg_store_real_remove_event (DVBEPGStore* base, guint event_id, DVBChannel* channel);
-static gboolean dvb_sqlite_epg_store_real_contains_event (DVBEPGStore* base, DVBEvent* event, DVBChannel* channel);
-static GeeList* dvb_sqlite_epg_store_real_get_events (DVBEPGStore* base, DVBChannel* channel);
+#define DVB_SQLITE_EPG_STORE_SELECT_EVENT_SQL "SELECT event_id, datetime(starttime),\n            duration, running_status, free_ca_mode, name,\n            description, extended_description\n            FROM events WHERE group_id=? AND sid=? AND event_id=?"
+static gboolean dvb_sqlite_epg_store_real_add_or_update_event (DVBEPGStore* base, DVBEvent* event, guint channel_sid, guint group_id);
+static DVBEvent* dvb_sqlite_epg_store_real_get_event (DVBEPGStore* base, guint event_id, guint channel_sid, guint group_id);
+static gboolean dvb_sqlite_epg_store_real_remove_event (DVBEPGStore* base, guint event_id, guint channel_sid, guint group_id);
+static gboolean dvb_sqlite_epg_store_real_contains_event (DVBEPGStore* base, DVBEvent* event, guint channel_sid, guint group_id);
+static GeeList* dvb_sqlite_epg_store_real_get_events (DVBEPGStore* base, guint channel_sid, guint group_id);
 static void _g_slist_free_dvb_event_audio_component_unref (GSList* self);
 static void _g_slist_free_dvb_event_video_component_unref (GSList* self);
 static void _g_slist_free_dvb_event_teletext_component_unref (GSList* self);
@@ -56,7 +72,7 @@ static void dvb_sqlite_epg_store_finalize (GObject* obj);
 
 
 
-static gboolean dvb_sqlite_epg_store_real_add_or_update_event (DVBEPGStore* base, DVBEvent* event, DVBChannel* channel) {
+static gboolean dvb_sqlite_epg_store_real_add_or_update_event (DVBEPGStore* base, DVBEvent* event, guint channel_sid, guint group_id) {
 	DVBSqliteEPGStore * self;
 	gint _tmp1;
 	gint free_ca_mode;
@@ -64,12 +80,11 @@ static gboolean dvb_sqlite_epg_store_real_add_or_update_event (DVBEPGStore* base
 	char* desc;
 	char* ext_desc;
 	double julian_start;
-	gboolean _tmp27;
+	gboolean _tmp31;
 	self = (DVBSqliteEPGStore*) base;
 	g_return_val_if_fail (event != NULL, FALSE);
-	g_return_val_if_fail (channel != NULL, FALSE);
 	if (self->priv->db == NULL) {
-		g_critical ("SqliteEPGStore.vala:80: SQLite error: No database connection");
+		g_critical ("SqliteEPGStore.vala:100: SQLite error: No database connection");
 		return FALSE;
 	}
 	_tmp1 = 0;
@@ -88,13 +103,16 @@ static gboolean dvb_sqlite_epg_store_real_add_or_update_event (DVBEPGStore* base
 		gboolean _tmp2;
 		return (_tmp2 = FALSE, name = (g_free (name), NULL), desc = (g_free (desc), NULL), ext_desc = (g_free (ext_desc), NULL), _tmp2);
 	}
-	if (dvb_epg_store_contains_event ((DVBEPGStore*) self, event, channel)) {
+	if (dvb_epg_store_contains_event ((DVBEPGStore*) self, event, channel_sid, group_id)) {
 		gboolean _tmp3;
 		gboolean _tmp4;
 		gboolean _tmp5;
 		gboolean _tmp6;
 		gboolean _tmp7;
 		gboolean _tmp8;
+		gboolean _tmp9;
+		gboolean _tmp10;
+		gboolean _tmp11;
 		sqlite3_reset (self->priv->update_event_statement);
 		_tmp3 = FALSE;
 		_tmp4 = FALSE;
@@ -102,156 +120,188 @@ static gboolean dvb_sqlite_epg_store_real_add_or_update_event (DVBEPGStore* base
 		_tmp6 = FALSE;
 		_tmp7 = FALSE;
 		_tmp8 = FALSE;
+		_tmp9 = FALSE;
+		_tmp10 = FALSE;
+		_tmp11 = FALSE;
 		if (sqlite3_bind_double (self->priv->update_event_statement, 1, julian_start) != SQLITE_OK) {
+			_tmp11 = TRUE;
+		} else {
+			_tmp11 = sqlite3_bind_int (self->priv->update_event_statement, 2, (gint) event->duration) != SQLITE_OK;
+		}
+		if (_tmp11) {
+			_tmp10 = TRUE;
+		} else {
+			_tmp10 = sqlite3_bind_int (self->priv->update_event_statement, 3, (gint) event->running_status) != SQLITE_OK;
+		}
+		if (_tmp10) {
+			_tmp9 = TRUE;
+		} else {
+			_tmp9 = sqlite3_bind_int (self->priv->update_event_statement, 4, free_ca_mode) != SQLITE_OK;
+		}
+		if (_tmp9) {
 			_tmp8 = TRUE;
 		} else {
-			_tmp8 = sqlite3_bind_int (self->priv->update_event_statement, 2, (gint) event->duration) != SQLITE_OK;
+			const char* _tmp12;
+			_tmp12 = NULL;
+			_tmp8 = sqlite3_bind_text (self->priv->update_event_statement, 5, (_tmp12 = name, (_tmp12 == NULL) ? NULL : g_strdup (_tmp12)), -1, g_free) != SQLITE_OK;
 		}
 		if (_tmp8) {
 			_tmp7 = TRUE;
 		} else {
-			_tmp7 = sqlite3_bind_int (self->priv->update_event_statement, 3, (gint) event->running_status) != SQLITE_OK;
+			const char* _tmp13;
+			_tmp13 = NULL;
+			_tmp7 = sqlite3_bind_text (self->priv->update_event_statement, 6, (_tmp13 = desc, (_tmp13 == NULL) ? NULL : g_strdup (_tmp13)), -1, g_free) != SQLITE_OK;
 		}
 		if (_tmp7) {
 			_tmp6 = TRUE;
 		} else {
-			_tmp6 = sqlite3_bind_int (self->priv->update_event_statement, 4, free_ca_mode) != SQLITE_OK;
+			const char* _tmp14;
+			_tmp14 = NULL;
+			_tmp6 = sqlite3_bind_text (self->priv->update_event_statement, 7, (_tmp14 = ext_desc, (_tmp14 == NULL) ? NULL : g_strdup (_tmp14)), -1, g_free) != SQLITE_OK;
 		}
 		if (_tmp6) {
 			_tmp5 = TRUE;
 		} else {
-			const char* _tmp9;
-			_tmp9 = NULL;
-			_tmp5 = sqlite3_bind_text (self->priv->update_event_statement, 5, (_tmp9 = name, (_tmp9 == NULL) ? NULL : g_strdup (_tmp9)), -1, g_free) != SQLITE_OK;
+			_tmp5 = sqlite3_bind_int (self->priv->update_event_statement, 8, (gint) group_id) != SQLITE_OK;
 		}
 		if (_tmp5) {
 			_tmp4 = TRUE;
 		} else {
-			const char* _tmp10;
-			_tmp10 = NULL;
-			_tmp4 = sqlite3_bind_text (self->priv->update_event_statement, 6, (_tmp10 = desc, (_tmp10 == NULL) ? NULL : g_strdup (_tmp10)), -1, g_free) != SQLITE_OK;
+			_tmp4 = sqlite3_bind_int (self->priv->update_event_statement, 9, (gint) channel_sid) != SQLITE_OK;
 		}
 		if (_tmp4) {
 			_tmp3 = TRUE;
 		} else {
-			const char* _tmp11;
-			_tmp11 = NULL;
-			_tmp3 = sqlite3_bind_text (self->priv->update_event_statement, 7, (_tmp11 = ext_desc, (_tmp11 == NULL) ? NULL : g_strdup (_tmp11)), -1, g_free) != SQLITE_OK;
+			_tmp3 = sqlite3_bind_int (self->priv->update_event_statement, 10, (gint) event->id) != SQLITE_OK;
 		}
 		if (_tmp3) {
-			gboolean _tmp12;
+			gboolean _tmp15;
 			dvb_sqlite_epg_store_print_last_error (self);
-			return (_tmp12 = FALSE, name = (g_free (name), NULL), desc = (g_free (desc), NULL), ext_desc = (g_free (ext_desc), NULL), _tmp12);
+			return (_tmp15 = FALSE, name = (g_free (name), NULL), desc = (g_free (desc), NULL), ext_desc = (g_free (ext_desc), NULL), _tmp15);
 		}
 		if (sqlite3_step (self->priv->update_event_statement) != SQLITE_DONE) {
-			gboolean _tmp13;
+			gboolean _tmp16;
 			dvb_sqlite_epg_store_print_last_error (self);
-			return (_tmp13 = FALSE, name = (g_free (name), NULL), desc = (g_free (desc), NULL), ext_desc = (g_free (ext_desc), NULL), _tmp13);
+			return (_tmp16 = FALSE, name = (g_free (name), NULL), desc = (g_free (desc), NULL), ext_desc = (g_free (ext_desc), NULL), _tmp16);
 		}
 	} else {
-		gboolean _tmp14;
-		gboolean _tmp15;
-		gboolean _tmp16;
 		gboolean _tmp17;
 		gboolean _tmp18;
 		gboolean _tmp19;
 		gboolean _tmp20;
 		gboolean _tmp21;
+		gboolean _tmp22;
+		gboolean _tmp23;
+		gboolean _tmp24;
+		gboolean _tmp25;
 		sqlite3_reset (self->priv->insert_event_statement);
-		_tmp14 = FALSE;
-		_tmp15 = FALSE;
-		_tmp16 = FALSE;
 		_tmp17 = FALSE;
 		_tmp18 = FALSE;
 		_tmp19 = FALSE;
 		_tmp20 = FALSE;
 		_tmp21 = FALSE;
-		if (sqlite3_bind_int (self->priv->insert_event_statement, 1, (gint) dvb_channel_get_Sid (channel)) != SQLITE_OK) {
+		_tmp22 = FALSE;
+		_tmp23 = FALSE;
+		_tmp24 = FALSE;
+		_tmp25 = FALSE;
+		if (sqlite3_bind_int (self->priv->insert_event_statement, 1, (gint) group_id) != SQLITE_OK) {
+			_tmp25 = TRUE;
+		} else {
+			_tmp25 = sqlite3_bind_int (self->priv->insert_event_statement, 2, (gint) channel_sid) != SQLITE_OK;
+		}
+		if (_tmp25) {
+			_tmp24 = TRUE;
+		} else {
+			_tmp24 = sqlite3_bind_int (self->priv->insert_event_statement, 3, (gint) event->id) != SQLITE_OK;
+		}
+		if (_tmp24) {
+			_tmp23 = TRUE;
+		} else {
+			_tmp23 = sqlite3_bind_double (self->priv->insert_event_statement, 4, julian_start) != SQLITE_OK;
+		}
+		if (_tmp23) {
+			_tmp22 = TRUE;
+		} else {
+			_tmp22 = sqlite3_bind_int (self->priv->insert_event_statement, 5, (gint) event->duration) != SQLITE_OK;
+		}
+		if (_tmp22) {
 			_tmp21 = TRUE;
 		} else {
-			_tmp21 = sqlite3_bind_int (self->priv->insert_event_statement, 2, (gint) event->id) != SQLITE_OK;
+			_tmp21 = sqlite3_bind_int (self->priv->insert_event_statement, 6, (gint) event->running_status) != SQLITE_OK;
 		}
 		if (_tmp21) {
 			_tmp20 = TRUE;
 		} else {
-			_tmp20 = sqlite3_bind_double (self->priv->insert_event_statement, 3, julian_start) != SQLITE_OK;
+			_tmp20 = sqlite3_bind_int (self->priv->insert_event_statement, 7, free_ca_mode) != SQLITE_OK;
 		}
 		if (_tmp20) {
 			_tmp19 = TRUE;
 		} else {
-			_tmp19 = sqlite3_bind_int (self->priv->insert_event_statement, 4, (gint) event->duration) != SQLITE_OK;
+			const char* _tmp26;
+			_tmp26 = NULL;
+			_tmp19 = sqlite3_bind_text (self->priv->insert_event_statement, 8, (_tmp26 = name, (_tmp26 == NULL) ? NULL : g_strdup (_tmp26)), -1, g_free) != SQLITE_OK;
 		}
 		if (_tmp19) {
 			_tmp18 = TRUE;
 		} else {
-			_tmp18 = sqlite3_bind_int (self->priv->insert_event_statement, 5, (gint) event->running_status) != SQLITE_OK;
+			const char* _tmp27;
+			_tmp27 = NULL;
+			_tmp18 = sqlite3_bind_text (self->priv->insert_event_statement, 9, (_tmp27 = desc, (_tmp27 == NULL) ? NULL : g_strdup (_tmp27)), -1, g_free) != SQLITE_OK;
 		}
 		if (_tmp18) {
 			_tmp17 = TRUE;
 		} else {
-			_tmp17 = sqlite3_bind_int (self->priv->insert_event_statement, 6, free_ca_mode) != SQLITE_OK;
+			const char* _tmp28;
+			_tmp28 = NULL;
+			_tmp17 = sqlite3_bind_text (self->priv->insert_event_statement, 10, (_tmp28 = ext_desc, (_tmp28 == NULL) ? NULL : g_strdup (_tmp28)), -1, g_free) != SQLITE_OK;
 		}
 		if (_tmp17) {
-			_tmp16 = TRUE;
-		} else {
-			const char* _tmp22;
-			_tmp22 = NULL;
-			_tmp16 = sqlite3_bind_text (self->priv->insert_event_statement, 7, (_tmp22 = name, (_tmp22 == NULL) ? NULL : g_strdup (_tmp22)), -1, g_free) != SQLITE_OK;
-		}
-		if (_tmp16) {
-			_tmp15 = TRUE;
-		} else {
-			const char* _tmp23;
-			_tmp23 = NULL;
-			_tmp15 = sqlite3_bind_text (self->priv->insert_event_statement, 8, (_tmp23 = desc, (_tmp23 == NULL) ? NULL : g_strdup (_tmp23)), -1, g_free) != SQLITE_OK;
-		}
-		if (_tmp15) {
-			_tmp14 = TRUE;
-		} else {
-			const char* _tmp24;
-			_tmp24 = NULL;
-			_tmp14 = sqlite3_bind_text (self->priv->insert_event_statement, 9, (_tmp24 = ext_desc, (_tmp24 == NULL) ? NULL : g_strdup (_tmp24)), -1, g_free) != SQLITE_OK;
-		}
-		if (_tmp14) {
-			gboolean _tmp25;
+			gboolean _tmp29;
 			dvb_sqlite_epg_store_print_last_error (self);
-			return (_tmp25 = FALSE, name = (g_free (name), NULL), desc = (g_free (desc), NULL), ext_desc = (g_free (ext_desc), NULL), _tmp25);
+			return (_tmp29 = FALSE, name = (g_free (name), NULL), desc = (g_free (desc), NULL), ext_desc = (g_free (ext_desc), NULL), _tmp29);
 		}
 		if (sqlite3_step (self->priv->insert_event_statement) != SQLITE_DONE) {
-			gboolean _tmp26;
+			gboolean _tmp30;
 			dvb_sqlite_epg_store_print_last_error (self);
-			return (_tmp26 = FALSE, name = (g_free (name), NULL), desc = (g_free (desc), NULL), ext_desc = (g_free (ext_desc), NULL), _tmp26);
+			return (_tmp30 = FALSE, name = (g_free (name), NULL), desc = (g_free (desc), NULL), ext_desc = (g_free (ext_desc), NULL), _tmp30);
 		}
 	}
-	return (_tmp27 = TRUE, name = (g_free (name), NULL), desc = (g_free (desc), NULL), ext_desc = (g_free (ext_desc), NULL), _tmp27);
+	return (_tmp31 = TRUE, name = (g_free (name), NULL), desc = (g_free (desc), NULL), ext_desc = (g_free (ext_desc), NULL), _tmp31);
 }
 
 
-static DVBEvent* dvb_sqlite_epg_store_real_get_event (DVBEPGStore* base, guint event_id, guint channel_sid) {
+static DVBEvent* dvb_sqlite_epg_store_real_get_event (DVBEPGStore* base, guint event_id, guint channel_sid, guint group_id) {
 	DVBSqliteEPGStore * self;
 	gboolean _tmp0;
+	gboolean _tmp1;
 	gint rc;
-	gboolean _tmp2;
+	gboolean _tmp3;
 	self = (DVBSqliteEPGStore*) base;
 	sqlite3_reset (self->priv->select_event_statement);
 	_tmp0 = FALSE;
-	if (sqlite3_bind_int (self->priv->select_event_statement, 1, (gint) channel_sid) != SQLITE_OK) {
+	_tmp1 = FALSE;
+	if (sqlite3_bind_int (self->priv->select_event_statement, 1, (gint) group_id) != SQLITE_OK) {
+		_tmp1 = TRUE;
+	} else {
+		_tmp1 = sqlite3_bind_int (self->priv->select_event_statement, 2, (gint) channel_sid) != SQLITE_OK;
+	}
+	if (_tmp1) {
 		_tmp0 = TRUE;
 	} else {
-		_tmp0 = sqlite3_bind_int (self->priv->select_event_statement, 2, (gint) event_id) != SQLITE_OK;
+		_tmp0 = sqlite3_bind_int (self->priv->select_event_statement, 3, (gint) event_id) != SQLITE_OK;
 	}
 	if (_tmp0) {
 		dvb_sqlite_epg_store_print_last_error (self);
 		return NULL;
 	}
 	rc = sqlite3_step (self->priv->select_event_statement);
-	_tmp2 = FALSE;
+	_tmp3 = FALSE;
 	if (rc != SQLITE_ROW) {
-		_tmp2 = rc != SQLITE_DONE;
+		_tmp3 = rc != SQLITE_DONE;
 	} else {
-		_tmp2 = FALSE;
+		_tmp3 = FALSE;
 	}
-	if (_tmp2) {
+	if (_tmp3) {
 		dvb_sqlite_epg_store_print_last_error (self);
 		return NULL;
 	}
@@ -264,21 +314,27 @@ static DVBEvent* dvb_sqlite_epg_store_real_get_event (DVBEPGStore* base, guint e
 }
 
 
-static gboolean dvb_sqlite_epg_store_real_remove_event (DVBEPGStore* base, guint event_id, DVBChannel* channel) {
+static gboolean dvb_sqlite_epg_store_real_remove_event (DVBEPGStore* base, guint event_id, guint channel_sid, guint group_id) {
 	DVBSqliteEPGStore * self;
 	gboolean _tmp1;
+	gboolean _tmp2;
 	self = (DVBSqliteEPGStore*) base;
-	g_return_val_if_fail (channel != NULL, FALSE);
 	if (self->priv->db == NULL) {
-		g_critical ("SqliteEPGStore.vala:160: SQLite error: No database connection");
+		g_critical ("SqliteEPGStore.vala:185: SQLite error: No database connection");
 		return FALSE;
 	}
 	sqlite3_reset (self->priv->delete_event_statement);
 	_tmp1 = FALSE;
-	if (sqlite3_bind_int (self->priv->delete_event_statement, 1, (gint) dvb_channel_get_Sid (channel)) != SQLITE_OK) {
+	_tmp2 = FALSE;
+	if (sqlite3_bind_int (self->priv->delete_event_statement, 1, (gint) group_id) != SQLITE_OK) {
+		_tmp2 = TRUE;
+	} else {
+		_tmp2 = sqlite3_bind_int (self->priv->delete_event_statement, 2, (gint) channel_sid) != SQLITE_OK;
+	}
+	if (_tmp2) {
 		_tmp1 = TRUE;
 	} else {
-		_tmp1 = sqlite3_bind_int (self->priv->delete_event_statement, 2, (gint) event_id) != SQLITE_OK;
+		_tmp1 = sqlite3_bind_int (self->priv->delete_event_statement, 3, (gint) event_id) != SQLITE_OK;
 	}
 	if (_tmp1) {
 		dvb_sqlite_epg_store_print_last_error (self);
@@ -292,19 +348,25 @@ static gboolean dvb_sqlite_epg_store_real_remove_event (DVBEPGStore* base, guint
 }
 
 
-static gboolean dvb_sqlite_epg_store_real_contains_event (DVBEPGStore* base, DVBEvent* event, DVBChannel* channel) {
+static gboolean dvb_sqlite_epg_store_real_contains_event (DVBEPGStore* base, DVBEvent* event, guint channel_sid, guint group_id) {
 	DVBSqliteEPGStore * self;
 	gboolean _tmp0;
+	gboolean _tmp1;
 	gint c;
 	self = (DVBSqliteEPGStore*) base;
 	g_return_val_if_fail (event != NULL, FALSE);
-	g_return_val_if_fail (channel != NULL, FALSE);
 	sqlite3_reset (self->priv->has_event_statement);
 	_tmp0 = FALSE;
-	if (sqlite3_bind_int (self->priv->has_event_statement, 1, (gint) dvb_channel_get_Sid (channel)) != SQLITE_OK) {
+	_tmp1 = FALSE;
+	if (sqlite3_bind_int (self->priv->has_event_statement, 1, (gint) group_id) != SQLITE_OK) {
+		_tmp1 = TRUE;
+	} else {
+		_tmp1 = sqlite3_bind_int (self->priv->has_event_statement, 2, (gint) channel_sid) != SQLITE_OK;
+	}
+	if (_tmp1) {
 		_tmp0 = TRUE;
 	} else {
-		_tmp0 = sqlite3_bind_int (self->priv->has_event_statement, 2, (gint) event->id) != SQLITE_OK;
+		_tmp0 = sqlite3_bind_int (self->priv->has_event_statement, 3, (gint) event->id) != SQLITE_OK;
 	}
 	if (_tmp0) {
 		dvb_sqlite_epg_store_print_last_error (self);
@@ -318,7 +380,7 @@ static gboolean dvb_sqlite_epg_store_real_contains_event (DVBEPGStore* base, DVB
 }
 
 
-static GeeList* dvb_sqlite_epg_store_real_get_events (DVBEPGStore* base, DVBChannel* channel) {
+static GeeList* dvb_sqlite_epg_store_real_get_events (DVBEPGStore* base, guint channel_sid, guint group_id) {
 	DVBSqliteEPGStore * self;
 	GeeList* events;
 	char* statement_str;
@@ -328,12 +390,11 @@ static GeeList* dvb_sqlite_epg_store_real_get_events (DVBEPGStore* base, DVBChan
 	sqlite3_stmt* _tmp1;
 	GeeList* _tmp5;
 	self = (DVBSqliteEPGStore*) base;
-	g_return_val_if_fail (channel != NULL, NULL);
 	events = (GeeList*) gee_array_list_new (DVB_TYPE_EVENT, (GBoxedCopyFunc) dvb_event_ref, dvb_event_unref, g_direct_equal);
 	if (self->priv->db == NULL) {
 		return events;
 	}
-	statement_str = g_strdup_printf (DVB_SQLITE_EPG_STORE_SELECT_ALL_EVENTS_STATEMENT, dvb_channel_get_Sid (channel));
+	statement_str = g_strdup_printf (DVB_SQLITE_EPG_STORE_SELECT_ALL_EVENTS_STATEMENT, group_id, channel_sid);
 	statement = NULL;
 	_tmp3 = NULL;
 	_tmp1 = NULL;
@@ -410,19 +471,19 @@ static DVBEvent* dvb_sqlite_epg_store_create_event_from_statement (DVBSqliteEPGS
 
 static void dvb_sqlite_epg_store_print_last_error (DVBSqliteEPGStore* self) {
 	g_return_if_fail (self != NULL);
-	g_critical ("SqliteEPGStore.vala:248: SQLite error: %d, %s", sqlite3_errcode (self->priv->db), sqlite3_errmsg (self->priv->db));
+	g_critical ("SqliteEPGStore.vala:275: SQLite error: %d, %s", sqlite3_errcode (self->priv->db), sqlite3_errmsg (self->priv->db));
 }
 
 
 static double dvb_sqlite_epg_store_to_julian (DVBSqliteEPGStore* self, guint year, guint month, guint day, guint hour, guint minute, guint second) {
 	char* datetime_str;
-	char* _tmp0;
+	const char* _tmp0;
 	double _tmp3;
 	g_return_val_if_fail (self != NULL, 0.0);
 	sqlite3_reset (self->priv->to_julian_statement);
 	datetime_str = g_strdup_printf ("%04u-%02u-%02u %02u:%02u:%02u", year, month, day, hour, minute, second);
 	_tmp0 = NULL;
-	if (sqlite3_bind_text (self->priv->to_julian_statement, 1, (_tmp0 = datetime_str, datetime_str = NULL, _tmp0), -1, g_free) != SQLITE_OK) {
+	if (sqlite3_bind_text (self->priv->to_julian_statement, 1, (_tmp0 = datetime_str, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), -1, g_free) != SQLITE_OK) {
 		double _tmp1;
 		dvb_sqlite_epg_store_print_last_error (self);
 		return (_tmp1 = (double) 0, datetime_str = (g_free (datetime_str), NULL), _tmp1);
@@ -455,13 +516,13 @@ static char* dvb_sqlite_epg_store_escape (const char* text) {
 		regex = (_tmp1 = g_regex_new ("'", G_REGEX_MULTILINE, 0, &inner_error), (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp1);
 		if (inner_error != NULL) {
 			if (inner_error->domain == G_REGEX_ERROR) {
-				goto __catch25_g_regex_error;
+				goto __catch35_g_regex_error;
 			}
-			goto __finally25;
+			goto __finally35;
 		}
 	}
-	goto __finally25;
-	__catch25_g_regex_error:
+	goto __finally35;
+	__catch35_g_regex_error:
 	{
 		GError * e;
 		e = inner_error;
@@ -469,13 +530,13 @@ static char* dvb_sqlite_epg_store_escape (const char* text) {
 		{
 			const char* _tmp2;
 			char* _tmp3;
-			g_warning ("SqliteEPGStore.vala:286: RegexError: %s", e->message);
+			g_warning ("SqliteEPGStore.vala:313: RegexError: %s", e->message);
 			_tmp2 = NULL;
 			_tmp3 = NULL;
 			return (_tmp3 = (_tmp2 = text, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp3);
 		}
 	}
-	__finally25:
+	__finally35:
 	if (inner_error != NULL) {
 		(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
 		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
@@ -489,15 +550,15 @@ static char* dvb_sqlite_epg_store_escape (const char* text) {
 		_tmp4 = g_regex_replace_literal (regex, text, strlen (text), 0, "''", 0, &inner_error);
 		if (inner_error != NULL) {
 			if (inner_error->domain == G_REGEX_ERROR) {
-				goto __catch26_g_regex_error;
+				goto __catch36_g_regex_error;
 			}
-			goto __finally26;
+			goto __finally36;
 		}
 		_tmp5 = NULL;
 		escaped_str = (_tmp5 = _tmp4, escaped_str = (g_free (escaped_str), NULL), _tmp5);
 	}
-	goto __finally26;
-	__catch26_g_regex_error:
+	goto __finally36;
+	__catch36_g_regex_error:
 	{
 		GError * e;
 		e = inner_error;
@@ -505,13 +566,13 @@ static char* dvb_sqlite_epg_store_escape (const char* text) {
 		{
 			const char* _tmp6;
 			char* _tmp7;
-			g_warning ("SqliteEPGStore.vala:295: RegexError: %s", e->message);
+			g_warning ("SqliteEPGStore.vala:322: RegexError: %s", e->message);
 			_tmp6 = NULL;
 			_tmp7 = NULL;
 			return (_tmp7 = (_tmp6 = text, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), escaped_str = (g_free (escaped_str), NULL), _tmp7);
 		}
 	}
-	__finally26:
+	__finally36:
 	if (inner_error != NULL) {
 		(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
 		escaped_str = (g_free (escaped_str), NULL);
@@ -544,24 +605,24 @@ static sqlite3* dvb_sqlite_epg_store_get_db_handler (void) {
 		{
 			dvb_utils_mkdirs (our_cache, &inner_error);
 			if (inner_error != NULL) {
-				goto __catch27_g_error;
-				goto __finally27;
+				goto __catch37_g_error;
+				goto __finally37;
 			}
 		}
-		goto __finally27;
-		__catch27_g_error:
+		goto __finally37;
+		__catch37_g_error:
 		{
 			GError * e;
 			e = inner_error;
 			inner_error = NULL;
 			{
 				sqlite3* _tmp0;
-				g_critical ("SqliteEPGStore.vala:312: Could not create directory: %s", e->message);
+				g_critical ("SqliteEPGStore.vala:339: Could not create directory: %s", e->message);
 				_tmp0 = NULL;
 				return (_tmp0 = NULL, (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), (cache_dir == NULL) ? NULL : (cache_dir = (g_object_unref (cache_dir), NULL)), (our_cache == NULL) ? NULL : (our_cache = (g_object_unref (our_cache), NULL)), (eventsdb == NULL) ? NULL : (eventsdb = (g_object_unref (eventsdb), NULL)), _tmp0);
 			}
 		}
-		__finally27:
+		__finally37:
 		if (inner_error != NULL) {
 			(cache_dir == NULL) ? NULL : (cache_dir = (g_object_unref (cache_dir), NULL));
 			(our_cache == NULL) ? NULL : (our_cache = (g_object_unref (our_cache), NULL));
@@ -592,7 +653,7 @@ static sqlite3* dvb_sqlite_epg_store_get_db_handler (void) {
 		val = (_tmp6 = sqlite3_exec (db, DVB_SQLITE_EPG_STORE_CREATE_EVENTS_TABLE_STATEMENT, NULL, NULL, &_tmp5), errormsg = (_tmp7 = _tmp5, errormsg = (g_free (errormsg), NULL), _tmp7), _tmp6);
 		if (val != SQLITE_OK) {
 			sqlite3* _tmp8;
-			g_critical ("SqliteEPGStore.vala:326: SQLite error: %s", errormsg);
+			g_critical ("SqliteEPGStore.vala:353: SQLite error: %s", errormsg);
 			_tmp8 = NULL;
 			return (_tmp8 = NULL, errormsg = (g_free (errormsg), NULL), (cache_dir == NULL) ? NULL : (cache_dir = (g_object_unref (cache_dir), NULL)), (our_cache == NULL) ? NULL : (our_cache = (g_object_unref (our_cache), NULL)), (eventsdb == NULL) ? NULL : (eventsdb = (g_object_unref (eventsdb), NULL)), (db == NULL) ? NULL : (db = (sqlite3_close (db), NULL)), _tmp8);
 		}
