@@ -1,3 +1,21 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <src/Timer.h>
 #include "src/Utils.h"
@@ -7,7 +25,7 @@
 
 struct _DVBTimerPrivate {
 	guint32 _Id;
-	guint _ChannelSid;
+	DVBChannel* _Channel;
 	char* _Name;
 	guint _Year;
 	guint _Month;
@@ -22,7 +40,7 @@ struct _DVBTimerPrivate {
 enum  {
 	DVB_TIMER_DUMMY_PROPERTY,
 	DVB_TIMER_ID,
-	DVB_TIMER_CHANNEL_SID,
+	DVB_TIMER_CHANNEL,
 	DVB_TIMER_NAME,
 	DVB_TIMER_YEAR,
 	DVB_TIMER_MONTH,
@@ -35,63 +53,36 @@ static DVBOverlapType dvb_timer_get_overlap (const time_t* this_start, const tim
 static time_t dvb_timer_get_end_time_timestamp (DVBTimer* self);
 static time_t dvb_timer_get_start_time_timestamp (DVBTimer* self);
 static void dvb_timer_set_Id (DVBTimer* self, guint32 value);
-static void dvb_timer_set_ChannelSid (DVBTimer* self, guint value);
-static void dvb_timer_set_Name (DVBTimer* self, const char* value);
-static void dvb_timer_set_Year (DVBTimer* self, guint value);
-static void dvb_timer_set_Month (DVBTimer* self, guint value);
-static void dvb_timer_set_Day (DVBTimer* self, guint value);
-static void dvb_timer_set_Hour (DVBTimer* self, guint value);
-static void dvb_timer_set_Minute (DVBTimer* self, guint value);
-static void dvb_timer_set_Duration (DVBTimer* self, guint value);
-static GObject * dvb_timer_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
+static void dvb_timer_set_Channel (DVBTimer* self, DVBChannel* value);
 static gpointer dvb_timer_parent_class = NULL;
 static void dvb_timer_finalize (GObject* obj);
 
 
 
-DVBTimer* dvb_timer_construct (GType object_type, guint32 id, guint channel_sid, gint year, gint month, gint day, gint hour, gint minute, guint duration, const char* name) {
+DVBTimer* dvb_timer_construct (GType object_type, guint32 id, DVBChannel* channel, gint year, gint month, gint day, gint hour, gint minute, guint duration, const char* name) {
 	GParameter * __params;
 	GParameter * __params_it;
 	DVBTimer * self;
-	__params = g_new0 (GParameter, 9);
+	g_return_val_if_fail (channel != NULL, NULL);
+	__params = g_new0 (GParameter, 2);
 	__params_it = __params;
 	__params_it->name = "Id";
 	g_value_init (&__params_it->value, G_TYPE_UINT);
 	g_value_set_uint (&__params_it->value, id);
 	__params_it++;
-	__params_it->name = "ChannelSid";
-	g_value_init (&__params_it->value, G_TYPE_UINT);
-	g_value_set_uint (&__params_it->value, channel_sid);
-	__params_it++;
-	__params_it->name = "Name";
-	g_value_init (&__params_it->value, G_TYPE_STRING);
-	g_value_set_string (&__params_it->value, name);
-	__params_it++;
-	__params_it->name = "Year";
-	g_value_init (&__params_it->value, G_TYPE_UINT);
-	g_value_set_uint (&__params_it->value, (guint) year);
-	__params_it++;
-	__params_it->name = "Month";
-	g_value_init (&__params_it->value, G_TYPE_UINT);
-	g_value_set_uint (&__params_it->value, (guint) month);
-	__params_it++;
-	__params_it->name = "Day";
-	g_value_init (&__params_it->value, G_TYPE_UINT);
-	g_value_set_uint (&__params_it->value, (guint) day);
-	__params_it++;
-	__params_it->name = "Hour";
-	g_value_init (&__params_it->value, G_TYPE_UINT);
-	g_value_set_uint (&__params_it->value, (guint) hour);
-	__params_it++;
-	__params_it->name = "Minute";
-	g_value_init (&__params_it->value, G_TYPE_UINT);
-	g_value_set_uint (&__params_it->value, (guint) minute);
-	__params_it++;
-	__params_it->name = "Duration";
-	g_value_init (&__params_it->value, G_TYPE_UINT);
-	g_value_set_uint (&__params_it->value, duration);
+	__params_it->name = "Channel";
+	g_value_init (&__params_it->value, DVB_TYPE_CHANNEL);
+	g_value_set_object (&__params_it->value, channel);
 	__params_it++;
 	self = g_object_newv (object_type, __params_it - __params, __params);
+	dvb_timer_set_Name (self, name);
+	dvb_timer_set_Year (self, (guint) year);
+	dvb_timer_set_Month (self, (guint) month);
+	dvb_timer_set_Day (self, (guint) day);
+	dvb_timer_set_Hour (self, (guint) hour);
+	dvb_timer_set_Minute (self, (guint) minute);
+	dvb_timer_set_Duration (self, duration);
+	self->priv->starttime = dvb_utils_create_time (year, month, day, hour, minute, 0);
 	while (__params_it > __params) {
 		--__params_it;
 		g_value_unset (&__params_it->value);
@@ -101,8 +92,8 @@ DVBTimer* dvb_timer_construct (GType object_type, guint32 id, guint channel_sid,
 }
 
 
-DVBTimer* dvb_timer_new (guint32 id, guint channel_sid, gint year, gint month, gint day, gint hour, gint minute, guint duration, const char* name) {
-	return dvb_timer_construct (DVB_TYPE_TIMER, id, channel_sid, year, month, day, hour, minute, duration, name);
+DVBTimer* dvb_timer_new (guint32 id, DVBChannel* channel, gint year, gint month, gint day, gint hour, gint minute, guint duration, const char* name) {
+	return dvb_timer_construct (DVB_TYPE_TIMER, id, channel, year, month, day, hour, minute, duration, name);
 }
 
 
@@ -112,21 +103,46 @@ DVBTimer* dvb_timer_new (guint32 id, guint channel_sid, gint year, gint month, g
 gboolean dvb_timer_conflicts_with (DVBTimer* self, DVBTimer* t2) {
 	time_t this_start;
 	time_t other_start;
+	DVBChannel* _tmp0;
+	DVBChannel* t1_c;
+	DVBChannel* _tmp1;
+	DVBChannel* t2_c;
+	gboolean _tmp2;
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (t2 != NULL, FALSE);
 	this_start = dvb_timer_get_start_time_timestamp (self);
 	other_start = dvb_timer_get_start_time_timestamp (t2);
+	_tmp0 = NULL;
+	t1_c = (_tmp0 = self->priv->_Channel, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
+	_tmp1 = NULL;
+	t2_c = (_tmp1 = t2->priv->_Channel, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
+	_tmp2 = FALSE;
+	if (dvb_channel_on_same_transport_stream (t1_c, t2_c)) {
+		_tmp2 = !dvb_channel_equals (t1_c, t2_c);
+	} else {
+		_tmp2 = FALSE;
+	}
+	if (_tmp2) {
+		gboolean _tmp3;
+		/* Recordings on the same TS never conflict as long
+		 as it isn't the same channel*/
+		return (_tmp3 = FALSE, (t1_c == NULL) ? NULL : (t1_c = (g_object_unref (t1_c), NULL)), (t2_c == NULL) ? NULL : (t2_c = (g_object_unref (t2_c), NULL)), _tmp3);
+	}
 	if (this_start <= other_start) {
 		time_t this_end;
+		gboolean _tmp4;
 		/* No conflict when this timer ends before other starts*/
 		this_end = dvb_timer_get_end_time_timestamp (self);
-		return this_end > other_start;
+		return (_tmp4 = this_end > other_start, (t1_c == NULL) ? NULL : (t1_c = (g_object_unref (t1_c), NULL)), (t2_c == NULL) ? NULL : (t2_c = (g_object_unref (t2_c), NULL)), _tmp4);
 	} else {
 		time_t other_end;
+		gboolean _tmp5;
 		/* No conflict when other timer ends before this starts*/
 		other_end = dvb_timer_get_end_time_timestamp (t2);
-		return other_end > this_start;
+		return (_tmp5 = other_end > this_start, (t1_c == NULL) ? NULL : (t1_c = (g_object_unref (t1_c), NULL)), (t2_c == NULL) ? NULL : (t2_c = (g_object_unref (t2_c), NULL)), _tmp5);
 	}
+	(t1_c == NULL) ? NULL : (t1_c = (g_object_unref (t1_c), NULL));
+	(t2_c == NULL) ? NULL : (t2_c = (g_object_unref (t2_c), NULL));
 }
 
 
@@ -204,6 +220,21 @@ static DVBOverlapType dvb_timer_get_overlap (const time_t* this_start, const tim
 }
 
 
+/**
+         * Add the specified amount of minutes to the starting time
+         */
+void dvb_timer_add_to_start_time (DVBTimer* self, gint minutes) {
+	g_return_if_fail (self != NULL);
+	self->priv->starttime.tm_min = self->priv->starttime.tm_min + minutes;
+	mktime (&self->priv->starttime);
+	dvb_timer_set_Year (self, (guint) (self->priv->starttime.tm_year + 1900));
+	dvb_timer_set_Month (self, (guint) (self->priv->starttime.tm_mon + 1));
+	dvb_timer_set_Day (self, (guint) self->priv->starttime.tm_mday);
+	dvb_timer_set_Hour (self, (guint) self->priv->starttime.tm_hour);
+	dvb_timer_set_Minute (self, (guint) self->priv->starttime.tm_min);
+}
+
+
 guint* dvb_timer_get_start_time (DVBTimer* self, int* result_length1) {
 	guint* _tmp1;
 	gint start_size;
@@ -240,49 +271,22 @@ guint* dvb_timer_get_end_time (DVBTimer* self, int* result_length1) {
 
 
 /**
-         * Whether the start time of the timer equals the current local time
+         * Whether the start time of the timer is after the current local time
+         * and the timer hasn't expired, yet.
          */
 gboolean dvb_timer_is_start_due (DVBTimer* self) {
-	time_t _tmp2 = {0};
 	time_t _tmp0 = {0};
-	struct tm _tmp1 = {0};
-	struct tm localtime;
-	gint year;
-	gint month;
-	gboolean _tmp3;
-	gboolean _tmp4;
-	gboolean _tmp5;
-	gboolean _tmp6;
+	time_t localtime;
+	gboolean _tmp1;
 	g_return_val_if_fail (self != NULL, FALSE);
-	localtime = (localtime_r ((_tmp2 = (time (&_tmp0), _tmp0), &_tmp2), &_tmp1), _tmp1);
-	/* Convert to values of struct tm aka Time            */
-	year = ((gint) self->priv->_Year) - 1900;
-	month = ((gint) self->priv->_Month) - 1;
-	_tmp3 = FALSE;
-	_tmp4 = FALSE;
-	_tmp5 = FALSE;
-	_tmp6 = FALSE;
-	if (year == localtime.tm_year) {
-		_tmp6 = month == localtime.tm_mon;
+	localtime = (time (&_tmp0), _tmp0);
+	_tmp1 = FALSE;
+	if ((localtime - mktime (&self->priv->starttime)) >= 0) {
+		_tmp1 = !dvb_timer_has_expired (self);
 	} else {
-		_tmp6 = FALSE;
+		_tmp1 = FALSE;
 	}
-	if (_tmp6) {
-		_tmp5 = self->priv->_Day == localtime.tm_mday;
-	} else {
-		_tmp5 = FALSE;
-	}
-	if (_tmp5) {
-		_tmp4 = self->priv->_Hour == localtime.tm_hour;
-	} else {
-		_tmp4 = FALSE;
-	}
-	if (_tmp4) {
-		_tmp3 = self->priv->_Minute == localtime.tm_min;
-	} else {
-		_tmp3 = FALSE;
-	}
-	return _tmp3;
+	return _tmp1;
 }
 
 
@@ -348,13 +352,13 @@ gboolean dvb_timer_has_expired (DVBTimer* self) {
 
 char* dvb_timer_to_string (DVBTimer* self) {
 	g_return_val_if_fail (self != NULL, NULL);
-	return g_strdup_printf ("channel: %u, start: %u-%u-%u %u:%u, duration: %u", self->priv->_ChannelSid, self->priv->_Year, self->priv->_Month, self->priv->_Day, self->priv->_Hour, self->priv->_Minute, self->priv->_Duration);
+	return g_strdup_printf ("channel: %u, start: %u-%u-%u %u:%u, duration: %u", dvb_channel_get_Sid (self->priv->_Channel), self->priv->_Year, self->priv->_Month, self->priv->_Day, self->priv->_Hour, self->priv->_Minute, self->priv->_Duration);
 }
 
 
 static time_t dvb_timer_get_end_time_timestamp (DVBTimer* self) {
 	struct tm t;
-	t = dvb_utils_create_time ((gint) self->priv->_Year, (gint) self->priv->_Month, (gint) self->priv->_Day, (gint) self->priv->_Hour, (gint) self->priv->_Minute, 0);
+	t = dvb_utils_create_time (self->priv->starttime.tm_year + 1900, self->priv->starttime.tm_mon + 1, self->priv->starttime.tm_mday, self->priv->starttime.tm_hour, self->priv->starttime.tm_min, 0);
 	/* TODO Do we change the value of this.starttime each time?*/
 	t.tm_min = t.tm_min + ((gint) self->priv->_Duration);
 	return mktime (&t);
@@ -381,16 +385,20 @@ static void dvb_timer_set_Id (DVBTimer* self, guint32 value) {
 }
 
 
-guint dvb_timer_get_ChannelSid (DVBTimer* self) {
-	g_return_val_if_fail (self != NULL, 0U);
-	return self->priv->_ChannelSid;
+DVBChannel* dvb_timer_get_Channel (DVBTimer* self) {
+	g_return_val_if_fail (self != NULL, NULL);
+	return self->priv->_Channel;
 }
 
 
-static void dvb_timer_set_ChannelSid (DVBTimer* self, guint value) {
+static void dvb_timer_set_Channel (DVBTimer* self, DVBChannel* value) {
+	DVBChannel* _tmp2;
+	DVBChannel* _tmp1;
 	g_return_if_fail (self != NULL);
-	self->priv->_ChannelSid = value;
-	g_object_notify ((GObject *) self, "ChannelSid");
+	_tmp2 = NULL;
+	_tmp1 = NULL;
+	self->priv->_Channel = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_Channel == NULL) ? NULL : (self->priv->_Channel = (g_object_unref (self->priv->_Channel), NULL)), _tmp2);
+	g_object_notify ((GObject *) self, "Channel");
 }
 
 
@@ -400,7 +408,7 @@ const char* dvb_timer_get_Name (DVBTimer* self) {
 }
 
 
-static void dvb_timer_set_Name (DVBTimer* self, const char* value) {
+void dvb_timer_set_Name (DVBTimer* self, const char* value) {
 	char* _tmp2;
 	const char* _tmp1;
 	g_return_if_fail (self != NULL);
@@ -417,7 +425,7 @@ guint dvb_timer_get_Year (DVBTimer* self) {
 }
 
 
-static void dvb_timer_set_Year (DVBTimer* self, guint value) {
+void dvb_timer_set_Year (DVBTimer* self, guint value) {
 	g_return_if_fail (self != NULL);
 	self->priv->_Year = value;
 	g_object_notify ((GObject *) self, "Year");
@@ -430,7 +438,7 @@ guint dvb_timer_get_Month (DVBTimer* self) {
 }
 
 
-static void dvb_timer_set_Month (DVBTimer* self, guint value) {
+void dvb_timer_set_Month (DVBTimer* self, guint value) {
 	g_return_if_fail (self != NULL);
 	self->priv->_Month = value;
 	g_object_notify ((GObject *) self, "Month");
@@ -443,7 +451,7 @@ guint dvb_timer_get_Day (DVBTimer* self) {
 }
 
 
-static void dvb_timer_set_Day (DVBTimer* self, guint value) {
+void dvb_timer_set_Day (DVBTimer* self, guint value) {
 	g_return_if_fail (self != NULL);
 	self->priv->_Day = value;
 	g_object_notify ((GObject *) self, "Day");
@@ -456,7 +464,7 @@ guint dvb_timer_get_Hour (DVBTimer* self) {
 }
 
 
-static void dvb_timer_set_Hour (DVBTimer* self, guint value) {
+void dvb_timer_set_Hour (DVBTimer* self, guint value) {
 	g_return_if_fail (self != NULL);
 	self->priv->_Hour = value;
 	g_object_notify ((GObject *) self, "Hour");
@@ -469,7 +477,7 @@ guint dvb_timer_get_Minute (DVBTimer* self) {
 }
 
 
-static void dvb_timer_set_Minute (DVBTimer* self, guint value) {
+void dvb_timer_set_Minute (DVBTimer* self, guint value) {
 	g_return_if_fail (self != NULL);
 	self->priv->_Minute = value;
 	g_object_notify ((GObject *) self, "Minute");
@@ -482,29 +490,13 @@ guint dvb_timer_get_Duration (DVBTimer* self) {
 }
 
 
-static void dvb_timer_set_Duration (DVBTimer* self, guint value) {
+void dvb_timer_set_Duration (DVBTimer* self, guint value) {
 	g_return_if_fail (self != NULL);
 	self->priv->_Duration = value;
 	g_object_notify ((GObject *) self, "Duration");
 }
 
 
-static GObject * dvb_timer_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
-	GObject * obj;
-	DVBTimerClass * klass;
-	GObjectClass * parent_class;
-	DVBTimer * self;
-	klass = DVB_TIMER_CLASS (g_type_class_peek (DVB_TYPE_TIMER));
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
-	self = DVB_TIMER (obj);
-	{
-		self->priv->starttime = dvb_utils_create_time ((gint) self->priv->_Year, (gint) self->priv->_Month, (gint) self->priv->_Day, (gint) self->priv->_Hour, (gint) self->priv->_Minute, 0);
-	}
-	return obj;
-}
-
-
 static void dvb_timer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
 	DVBTimer * self;
 	gpointer boxed;
@@ -513,8 +505,8 @@ static void dvb_timer_get_property (GObject * object, guint property_id, GValue
 		case DVB_TIMER_ID:
 		g_value_set_uint (value, dvb_timer_get_Id (self));
 		break;
-		case DVB_TIMER_CHANNEL_SID:
-		g_value_set_uint (value, dvb_timer_get_ChannelSid (self));
+		case DVB_TIMER_CHANNEL:
+		g_value_set_object (value, dvb_timer_get_Channel (self));
 		break;
 		case DVB_TIMER_NAME:
 		g_value_set_string (value, dvb_timer_get_Name (self));
@@ -551,8 +543,8 @@ static void dvb_timer_set_property (GObject * object, guint property_id, const G
 		case DVB_TIMER_ID:
 		dvb_timer_set_Id (self, g_value_get_uint (value));
 		break;
-		case DVB_TIMER_CHANNEL_SID:
-		dvb_timer_set_ChannelSid (self, g_value_get_uint (value));
+		case DVB_TIMER_CHANNEL:
+		dvb_timer_set_Channel (self, g_value_get_object (value));
 		break;
 		case DVB_TIMER_NAME:
 		dvb_timer_set_Name (self, g_value_get_string (value));
@@ -587,17 +579,16 @@ static void dvb_timer_class_init (DVBTimerClass * klass) {
 	g_type_class_add_private (klass, sizeof (DVBTimerPrivate));
 	G_OBJECT_CLASS (klass)->get_property = dvb_timer_get_property;
 	G_OBJECT_CLASS (klass)->set_property = dvb_timer_set_property;
-	G_OBJECT_CLASS (klass)->constructor = dvb_timer_constructor;
 	G_OBJECT_CLASS (klass)->finalize = dvb_timer_finalize;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_ID, g_param_spec_uint ("Id", "Id", "Id", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_CHANNEL_SID, g_param_spec_uint ("ChannelSid", "ChannelSid", "ChannelSid", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_NAME, g_param_spec_string ("Name", "Name", "Name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_YEAR, g_param_spec_uint ("Year", "Year", "Year", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_MONTH, g_param_spec_uint ("Month", "Month", "Month", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_DAY, g_param_spec_uint ("Day", "Day", "Day", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_HOUR, g_param_spec_uint ("Hour", "Hour", "Hour", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_MINUTE, g_param_spec_uint ("Minute", "Minute", "Minute", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_DURATION, g_param_spec_uint ("Duration", "Duration", "Duration", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_CHANNEL, g_param_spec_object ("Channel", "Channel", "Channel", DVB_TYPE_CHANNEL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_NAME, g_param_spec_string ("Name", "Name", "Name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_YEAR, g_param_spec_uint ("Year", "Year", "Year", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_MONTH, g_param_spec_uint ("Month", "Month", "Month", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_DAY, g_param_spec_uint ("Day", "Day", "Day", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_HOUR, g_param_spec_uint ("Hour", "Hour", "Hour", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_MINUTE, g_param_spec_uint ("Minute", "Minute", "Minute", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_TIMER_DURATION, g_param_spec_uint ("Duration", "Duration", "Duration", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
 
 
@@ -609,6 +600,7 @@ static void dvb_timer_instance_init (DVBTimer * self) {
 static void dvb_timer_finalize (GObject* obj) {
 	DVBTimer * self;
 	self = DVB_TIMER (obj);
+	(self->priv->_Channel == NULL) ? NULL : (self->priv->_Channel = (g_object_unref (self->priv->_Channel), NULL));
 	self->priv->_Name = (g_free (self->priv->_Name), NULL);
 	G_OBJECT_CLASS (dvb_timer_parent_class)->finalize (obj);
 }
